import math


def get_current_estimate(r):
    """
    Gets the value of r * r * pi for the current estimate of r.
    r should be a number and the return value will also be a number.
    """
    return r * r * math.pi


def get_midpoint(lower, upper):
    """
    Returns the midpoint of lower and upper.
    They should both be numbers - return value will be a float.
    """
    return (lower + upper) / 2


def calculate_r(lower, upper):
    """
    Recursively calculates the number r such that r * r * pi = 1
    Lower and upper are the lower and upper bounds of the interval to search in.
    Returns a floating point number.
    """

    midpoint = get_midpoint(lower, upper)
    current_estimate = get_current_estimate(midpoint)

    # Base case - we're close enough now
    if abs(current_estimate - 1) < 1e-6:  # abs is a function to calculate absolute value
        return midpoint
    # Midpoint is too high - use it as the new upper bound
    elif current_estimate - 1 > 0:
        return calculate_r(lower, midpoint)
    # Midpoint must be too low - use it as the new lower bound
    else:
        return calculate_r(midpoint, upper)

#print(calculate_r(0, 1))


def calculate_r_verbose(lower, upper, recursion_number):
    """
    Recursively calculates the number r such that r * r * pi = 1
    Lower and upper are the lower and upper bounds of the interval to search in.
    Returns a floating point number.
    Identical to the above function but prints a lot more information to the user.
    """
    recursion_number += 1
    print("Called the function {} times".format(recursion_number))

    midpoint = get_midpoint(lower, upper)
    current_estimate = get_current_estimate(midpoint)

    print("Current estimate is {}".format(current_estimate))

    # Base case - we're close enough now
    if abs(current_estimate - 1) < 1e-6:  # abs is a function to calculate absolute value
        print("Estimate is close enough to 1 - ceasing recursion")
        return midpoint
    # Midpoint is too high - use it as the new upper bound
    elif current_estimate - 1 > 0:
        print("Estimate is too high - calling again with new lower bound")
        final_estimate = calculate_r_verbose(lower, midpoint, recursion_number)
        print("Exiting recursion depth {}".format(recursion_number))
        return final_estimate
    # Midpoint must be too low - use it as the new lower bound
    else:
        print("Estimate is too low - calling again with new upper bound")
        final_estimate = calculate_r_verbose(midpoint, upper, recursion_number)
        print("Exiting recursion depth {}".format(recursion_number))
        return final_estimate

# print(calculate_r_verbose(0, 1, 0))